package com.tencent.xmagic;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.FrameLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.constraintlayout.widget.ConstraintLayout;


import com.tencent.xmagic.download.DownloadErrorCode;
import com.tencent.xmagic.download.FileUtil;
import com.tencent.xmagic.download.MotionDLModel;
import com.tencent.xmagic.download.ResDownloadConfig;
import com.tencent.xmagic.download.ResDownloadUtil;
import com.tencent.xmagic.module.XmagicResParser;
import com.tencent.xmagic.utils.OnDownloadListener;
import com.tencent.xmagic.utils.ScreenUtils;
import com.tencent.xmagic.demo.R;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class XmagicLoadAssetsView {
    private              Activity         mAct;
    private final        ViewGroup        mainView;
    private static final int              MSG_UPDATE_PROGRESS = 0;
    private              ConstraintLayout mLayoutDownload;
    private              TextView         mTextDownloadLibs, mTextDownloadAssets, mTextDownloadMotion;
    private Button  mBtnFinishLoadAssets;
    private boolean mUseLocalRes = true;
    private boolean mIsDownloadLibs, mIsDownloadAssets, mIsDownloadMotion;

    /**
     * 表示资源是否已被复制
     */
    public static boolean isCopyedRes = false;


    private Handler mHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_UPDATE_PROGRESS:
                    if (msg.arg1 == ResDownloadUtil.FILE_TYPE_LIBS) {
                        mTextDownloadLibs.setText("libs下载中:" + (String) msg.obj);
                    } else if (msg.arg1 == ResDownloadUtil.FILE_TYPE_ASSETS) {
                        mTextDownloadAssets.setText("assets下载中:" + (String) msg.obj);
                    }
                    break;
            }
        }
    };

    /**
     * @param act 布局根view
     */
    public XmagicLoadAssetsView(Activity act) {
        if (act == null)
            throw new IllegalStateException("activity is null!!");
        this.mAct = act;
        View view = View.inflate(act, R.layout.xmagic_load_assets_view, null);
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        view.setBackgroundColor(Color.WHITE);
        act.getWindow().addContentView(view, params);
        this.mainView = (ViewGroup) view;
        initView();
    }

    public void destroyView() {
        if (mOnAssetsLoadFinishListener != null) {
            mOnAssetsLoadFinishListener.onAssetsLoadFinish();
        }
        ViewGroup view = (ViewGroup) mainView.getParent();
        view.removeView(mainView);
        mAct = null;
    }

    private void initView() {
        mTextDownloadLibs = mainView.findViewById(R.id.tv_download_libs);
        mTextDownloadAssets = mainView.findViewById(R.id.tv_download_assets);
        mLayoutDownload = mainView.findViewById(R.id.layout_download);
        mTextDownloadLibs = mainView.findViewById(R.id.tv_download_libs);
        mTextDownloadMotion = mainView.findViewById(R.id.tv_download_motion);
        mBtnFinishLoadAssets = mainView.findViewById(R.id.btn_finish_load_assets);
        RadioGroup resLocationGroup = mainView.findViewById(R.id.resLocationGroup);
        RadioButton radioLocal = mainView.findViewById(R.id.radio_use_local);
        RadioButton radioDownload = mainView.findViewById(R.id.radio_use_download);
        initRadioGroup(resLocationGroup);
        mBtnFinishLoadAssets.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!mUseLocalRes) {
                    String validLibsDirectory = ResDownloadUtil.getValidLibsDirectory(mAct,
                            isCpuV8a() ? ResDownloadConfig.DOWNLOAD_MD5_LIBS_V8A : ResDownloadConfig.DOWNLOAD_MD5_LIBS_V7A);
                    if (validLibsDirectory == null) {
                        Toast.makeText(mAct, "libs没有下载好，请先下载", Toast.LENGTH_LONG).show();
                        return;
                    }
                    XmagicApi.setLibPathAndLoad(validLibsDirectory);
                    String validAssetsDirectory = ResDownloadUtil.getValidAssetsDirectory(mAct, ResDownloadConfig.DOWNLOAD_MD5_ASSETS);
                    if (validAssetsDirectory == null) {
                        Toast.makeText(mAct, "assets没有下载好，请先下载", Toast.LENGTH_LONG).show();
                        return;
                    }
                }
                destroyView();
            }
        });

        radioLocal.setOnClickListener(v -> {
            mBtnFinishLoadAssets.setVisibility(View.GONE);
            mUseLocalRes = true;
            mLayoutDownload.setVisibility(View.GONE);
            XmagicResParser.setResPath(new File(mAct.getFilesDir(), "xmagic").getAbsolutePath());
            AlertDialog alertDialog = new ProgressDialog.Builder(mAct).setTitle("资源拷贝中...").setCancelable(false).create();
            alertDialog.show();
            new Thread() {
                @Override
                public void run() {
                    XmagicResParser.copyRes(mAct.getApplicationContext());
                    mAct.runOnUiThread(() -> {
                        alertDialog.dismiss();
                        isCopyedRes = true;
                        mBtnFinishLoadAssets.setVisibility(View.VISIBLE);
                    });
                }
            }.start();
        });
        radioDownload.setOnClickListener(v -> {
            mBtnFinishLoadAssets.setVisibility(View.GONE);
            Dialog dialog = new AlertDialog.Builder(mAct)
                    .setTitle("提示")
                    .setMessage("1，把assets、libs、动效资源的zip包上传到云端，生成下载URL\n\n" +
                            "2,更新demo工程的ResDownloadConfig.java里的URL和MD5\n\n" +
                            "3,如果切换了加载方式，请卸载重装demo\n\n" +
                            "4,更详细的指引，请参考SDK/文档/doc.html/Android接入指引 ")
                    .setPositiveButton("我已完成上述操作", (dialog1, which) -> {
                        dialog1.dismiss();
                        mUseLocalRes = false;
                        mLayoutDownload.setVisibility(View.VISIBLE);
                    })
                    .setNegativeButton("取消", (dialog12, which) -> {
                        dialog12.dismiss();
                        radioLocal.setChecked(true);
                    })
                    .setOnCancelListener(dialog13 -> {
                        radioLocal.setChecked(true);
                    })
                    .create();
            dialog.show();
        });

        mainView.findViewById(R.id.btn_check_download).setOnClickListener(v -> {
            if (FileUtil.getAvailableInternalMemorySize() < 100 * 1024 * 1024) {
                mTextDownloadLibs.setText("空间不足，无法下载libs");
            } else {
                if (isCpuV8a()) {
                    checkDownload(ResDownloadUtil.FILE_TYPE_LIBS, ResDownloadConfig.DOWNLOAD_URL_LIBS_V8A, ResDownloadConfig.DOWNLOAD_MD5_LIBS_V8A);
                } else {
                    checkDownload(ResDownloadUtil.FILE_TYPE_LIBS, ResDownloadConfig.DOWNLOAD_URL_LIBS_V7A, ResDownloadConfig.DOWNLOAD_MD5_LIBS_V7A);
                }
            }
        });
        mainView.findViewById(R.id.btn_check_download_assets).setOnClickListener(v -> {
            if (FileUtil.getAvailableInternalMemorySize() < 100 * 1024 * 1024) {
                mTextDownloadAssets.setText("空间不足，无法下载assets");
            } else {
                checkDownload(ResDownloadUtil.FILE_TYPE_ASSETS, ResDownloadConfig.DOWNLOAD_URL_ASSETS, ResDownloadConfig.DOWNLOAD_MD5_ASSETS);
            }
        });

        mainView.findViewById(R.id.btn_check_download_motion).setOnClickListener(v -> {
            if (FileUtil.getAvailableInternalMemorySize() < 100 * 1024 * 1024) {
                mTextDownloadMotion.setText("空间不足，无法下载动效");
            } else {
                checkDownloadMotion();
            }
        });

        boolean enableResumeBreakPoint = mAct.getSharedPreferences("demo_settings", Context.MODE_PRIVATE)
                .getBoolean("ENABLE_RESUME_BREAK_POINT", true);
        ResDownloadUtil.ENABLE_RESUME_FROM_BREAKPOINT = enableResumeBreakPoint;

        CheckBox checkBoxResumeBreakPoint = mainView.findViewById(R.id.checkbox_resume_breakpoint);
        checkBoxResumeBreakPoint.setChecked(enableResumeBreakPoint);
        checkBoxResumeBreakPoint.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                SharedPreferences sp = mAct.getSharedPreferences("demo_settings", Context.MODE_PRIVATE);
                sp.edit().putBoolean("ENABLE_RESUME_BREAK_POINT", isChecked).apply();
                ResDownloadUtil.ENABLE_RESUME_FROM_BREAKPOINT = isChecked;
            }
        });

    }

    private boolean isCpuV8a() {
        String cpuType = null;
        try {
            cpuType = new BufferedReader(new InputStreamReader(Runtime.getRuntime().exec(
                    "getprop ro.product.cpu.abi").getInputStream())).readLine();
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (cpuType != null && cpuType.contains("arm64-v8a")) {
            return true;
        }
        return false;
    }

    private void checkDownload(int fileType, String downloadUrl, String downloadMd5) {
        ResDownloadUtil.checkOrDownloadFiles(mAct, fileType, downloadUrl, downloadMd5, new OnDownloadListener() {
            @Override
            public void onDownloadSuccess(String directory) {
                mAct.runOnUiThread(() -> {
                    if (fileType == ResDownloadUtil.FILE_TYPE_LIBS) {
                        mTextDownloadLibs.setText("libs下载成功");
                        mIsDownloadLibs = true;
                        isDownloadFinish();
                    } else if (fileType == ResDownloadUtil.FILE_TYPE_ASSETS) {
                        mTextDownloadAssets.setText("assets下载成功");
                        mIsDownloadAssets = true;
                        isDownloadFinish();
                    }
                });
            }

            @Override
            public void onDownloading(int progress) {
                Message msg = mHandler.obtainMessage(MSG_UPDATE_PROGRESS);
                msg.arg1 = fileType;
                msg.obj = progress + "%";
                mHandler.sendMessage(msg);
            }

            @Override
            public void onDownloadFailed(int errorCode) {
                mAct.runOnUiThread(() -> {
                    TextView textView = null;
                    String textPrefix = null;
                    if (fileType == ResDownloadUtil.FILE_TYPE_LIBS) {
                        textView = mTextDownloadLibs;
                        textPrefix = "libs";
                    } else if (fileType == ResDownloadUtil.FILE_TYPE_ASSETS) {
                        textView = mTextDownloadAssets;
                        textPrefix = "assets";
                    }
                    if (textView == null) {
                        return;
                    }
                    textView.setText(textPrefix + "下载失败：(" + errorCode + ")" + DownloadErrorCode.getErrorMsg(errorCode));
                });
            }
        });
    }

    private boolean      isDownloadingMotions       = false;
    private int          downloadMotionFinishCount  = 0;
    private int          downloadMotionSuccessCount = 0;
    private int          downloadMotionFailCount    = 0;
    private List<String> motionSuccessList          = new ArrayList<>();
    private List<String> motionFailList             = new ArrayList<>();

    private void checkDownloadMotion() {
        if (isDownloadingMotions) {
            return;
        }
        isDownloadingMotions = true;
        downloadMotionFinishCount = 0;
        downloadMotionSuccessCount = 0;
        downloadMotionFailCount = 0;
        motionSuccessList.clear();
        motionFailList.clear();

        List<MotionDLModel> motionList = ResDownloadConfig.getMotionList();
        downloadOneByOne(motionList, 0);
    }

    private void downloadOneByOne(List<MotionDLModel> motionList, int index) {
        String text = mTextDownloadMotion.getText().toString();
        if (text == null) {
            text = "";
        }
        text = text + "\n" + "正在下载动效：" + motionList.get(index).getName();
        mTextDownloadMotion.setText(text);
        ResDownloadUtil.checkOrDownloadMotions(mAct, motionList.get(index), new OnDownloadListener() {

            @Override
            public void onDownloadSuccess(String motionName) {
                mAct.runOnUiThread(() -> {
                    motionSuccessList.add(motionName + "\r\r成功");
                    downloadMotionFinishCount++;
                    downloadMotionSuccessCount++;
                    updateTextAndDownloadNextOne();
                });
            }

            @Override
            public void onDownloading(int progress) {
            }

            @Override
            public void onDownloadFailed(int errorCode) {
                mAct.runOnUiThread(() -> {
                    motionFailList.add(motionList.get(index).getName() + "\r\r失败：" + DownloadErrorCode.getErrorMsg(errorCode));
                    downloadMotionFinishCount++;
                    downloadMotionFailCount++;
                    updateTextAndDownloadNextOne();
                });
            }

            private void updateTextAndDownloadNextOne() {
                mTextDownloadMotion.setText("动效下载中，成功" + downloadMotionSuccessCount + "/" + motionList.size() + "，失败" + downloadMotionFailCount + "个");
                if (downloadMotionFinishCount < motionList.size()) {
                    downloadOneByOne(motionList, index + 1);
                } else {
                    mTextDownloadMotion.setText("动效下载完成：成功" + downloadMotionSuccessCount + "个，失败" + downloadMotionFailCount + "个");
                    isDownloadingMotions = false;
                    showSummaryDialog();
                    mIsDownloadMotion = true;
                    isDownloadFinish();
                }
            }

            private void showSummaryDialog() {
                StringBuilder sb = new StringBuilder();
                for (String str : motionFailList) {
                    sb.append(str).append("\n");
                }
                for (String str : motionSuccessList) {
                    sb.append(str).append("\n");
                }

                Dialog dialog = new AlertDialog.Builder(mAct)
                        .setTitle("动效下载结果")
                        .setMessage(sb.toString())
                        .setNegativeButton(null, null)
                        .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }).create();
                dialog.show();
            }
        });
    }

    private void initRadioGroup(RadioGroup radioGroup) {
        int childCount = radioGroup.getChildCount();
        int itemViewWidth = (ScreenUtils.getWindowsWidth(mAct) - ScreenUtils.dip2px(mAct, 52) - 28) / childCount;
        for (int i = 0; i < childCount; i++) {
            View childAt = radioGroup.getChildAt(i);
            RadioGroup.LayoutParams layoutParams = (RadioGroup.LayoutParams) childAt.getLayoutParams();
            layoutParams.width = itemViewWidth;
            if (i != 0) {
                layoutParams.setMargins(12, 0, 0, 0);
            }
            childAt.setLayoutParams(layoutParams);
        }
    }

    private void isDownloadFinish() {
        if (mIsDownloadMotion && mIsDownloadLibs && mIsDownloadAssets) {
            mBtnFinishLoadAssets.setVisibility(View.VISIBLE);
        }
    }

    private OnAssetsLoadFinishListener mOnAssetsLoadFinishListener;

    public void setOnAssetsLoadFinishListener(OnAssetsLoadFinishListener ls) {
        this.mOnAssetsLoadFinishListener = ls;
    }

    public interface OnAssetsLoadFinishListener {
        void onAssetsLoadFinish();
    }
}
